/*
 * Most of the work is done in this class, including building the interval graph
 * Author: Jenna Butler
 */

//package jennabutler.hdcnv;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.*;

import org.jgrapht.alg.*;
import org.jgrapht.*;
import org.jgrapht.graph.*;

import java.io.*;
import org.jgrapht.ext.*;




public class BuildGraph {

	//Global graph
	public static UndirectedGraph<CNV, DefaultEdge> CNVGraph;
	public static int[][] adjacencyMatrix;
	public static int numOfNodes;
	public static int numOfColumns;
	//public static int extraColumns;
	public static double overlapPercent, familyPercent;
	public static boolean graphFiles, trackFiles;


	//This method needs to analyze all the CNVs and see if they have overlap with anything else...
	//Use 2 iterators?
	/*
	 * Method to build a graph from an existing file of CNV data
	 * @param fileName where the data is stored
	 * @param overlapPercent: how much 2 CNV events have to overlap to count it in a merge
	 * @param familyPercent: How much two CNV events have to overlap to put them in the same family
	 */
	public static void buildGraphFromFile(String fileName) throws HDCNVInputException{
		//Each time you build a graph, make a new graph
		CNVGraph = new SimpleGraph<CNV, DefaultEdge>(DefaultEdge.class);
		//Also reset number of nodes for adjacency matrix
		numOfNodes= 0;

	
		//Parse the file to add vertices to the graph
		//parseFile(fileName);
		//JENNA TEST THIS
		FormatInput.parseFile(fileName);

	
		//Create a new adjancecy matrix for each file.
		adjacencyMatrix = new int[numOfNodes][numOfNodes];

		double overlapMultiplier = overlapPercent/100;
		double familyMultiplier = familyPercent/100;
		double outerOverlapAmount, innerOverlapAmount;
		double familyOverlapAmountOuter, familyOverlapAmountInner;
		//Get the vertex set, and get an iterator for each
		//Probably could use just 1 vertex set but I wans't positive... mark test
		Set<CNV> vertexSet1 = CNVGraph.vertexSet();
		Set<CNV> vertexSet2 = CNVGraph.vertexSet();
		Iterator<CNV> outerIt = vertexSet1.iterator();
		Iterator<CNV> innerIt = vertexSet2.iterator();

		//Make some variables to use in the loop
		CNV currentOuter = null;
		CNV currentInner = null;
		int outerEnd, innerStart, outerLength, innerLength, sharedLength;
		//Loop through the outer comparing it to all the inners... n2 comparasions
		while (outerIt.hasNext()){
			innerIt = vertexSet2.iterator();
			currentOuter = outerIt.next();
			outerEnd = currentOuter.getEnd();
			outerLength = currentOuter.getLength();
			while (innerIt.hasNext()){
				currentInner = innerIt.next();
				innerStart = currentInner.getStart();
				innerLength = currentInner.getLength();
				//Check if there is any overlap at all
				if (currentInner.getID() > currentOuter.getID()){
					if (innerStart < outerEnd){
						//Get the length they share
						sharedLength = outerEnd - innerStart;
						//Check if the length they share is >= the minimum shared lneght entered by the user
						outerOverlapAmount = (outerLength*overlapMultiplier);
						innerOverlapAmount = (innerLength*overlapMultiplier);
						if ((sharedLength >= (outerOverlapAmount)) || (sharedLength >= (innerOverlapAmount))){
							//If the shared length is enough, make sure the edge isn't already in the graph (test mark: do I have to do both?)
							if (!(CNVGraph.containsEdge(currentOuter, currentInner))){
								if (!(CNVGraph.containsEdge(currentInner, currentOuter))){
									addEdge(currentInner, currentOuter);
								}
								//If they are family memebers, add the inner CNV to the family members list
								familyOverlapAmountOuter = outerLength*familyMultiplier;
								familyOverlapAmountInner = innerLength*familyMultiplier;
								if ((sharedLength >= familyOverlapAmountOuter) || (sharedLength >= familyOverlapAmountInner)){
									addFamily(currentOuter, currentInner);
								}
							}
						}
					}
				}
			}
		}
		//Now that you have a graph, you want to find all of the families and all of the merge regions.
	}

	/*
	 * Helper method to add edge and update matrix
	 */
	private static void addEdge(CNV currentOuter, CNV currentInner){
		//If the edge isn't already in the graph, add it
		CNVGraph.addEdge(currentOuter, currentInner);
		//Update the adjacency matrix - I am storing it symmetrically for other methods
		adjacencyMatrix[currentOuter.getID()-1][currentInner.getID()-1] = 1;
		adjacencyMatrix[currentInner.getID()-1][currentOuter.getID()-1] = 1;
	}

	/*
	 * Add a family member 
	 */
	private static void addFamily(CNV currentOuter, CNV currentInner){
		if (currentOuter.family == null){
			currentOuter.initalizeFamily();
		}
		currentOuter.addFamilyMember(currentInner.getID());
		if (currentInner.family == null){
			currentInner.initalizeFamily();
		}
		currentInner.addFamilyMember(currentOuter.getID());
	}

	/*
	 * Reads in the initial input file
	 * Builds a vertex for each CNV in the file 
	 * Does this to the global graph
	 */
	public static void parseFile(String fileName){
	
		BufferedReader inFileReader = null; 
		String line = null;
		//Open the file and read the first line
		try {
			inFileReader = new BufferedReader(new FileReader(fileName));
			line = inFileReader.readLine();
			line = inFileReader.readLine(); //first line is headers so skip that line
		}
		catch(Exception e){
			System.out.println("Error :" + e.getMessage());
		}
		//CNV[] cnvs = new CNV[60];


		//loop through the file and create the CNV objects
		while (line != null){
			StringTokenizer st = new StringTokenizer(line, ",");
			//while (st.hasMoreTokens()){
			CNV temp = new CNV();
			String next = st.nextToken();
			
			//order of the file should be ID, chromo, start, end, sample, cntype, length, makers, pvalue, known snps
			temp.setID(Integer.parseInt(next));
			temp.setChromo(st.nextToken());
			temp.setStart(Integer.parseInt(st.nextToken()));
			temp.setEnd(Integer.parseInt(st.nextToken()));
			temp.setSampleID(Integer.parseInt(st.nextToken()));
			temp.setCNType(st.nextToken());
			temp.setLength(Integer.parseInt(st.nextToken()));
			temp.setMarkers(Integer.parseInt(st.nextToken()));
			temp.setpValue(Double.parseDouble(st.nextToken()));
			temp.setKnownSNPs(Integer.parseInt(st.nextToken()));
			temp.setTissueAndGeno();
			
			CNVGraph.addVertex(temp);
			numOfNodes++;
			//}
			try {
				line = inFileReader.readLine();
			}
			catch (Exception e){
				System.out.println("Error: " + e.getMessage());
			}
		}
		try{
			inFileReader.close();
		}
		catch (Exception e){
			e.printStackTrace();
		}
	}

	/*
	 * This method is called by the GUI
	 * It takes a directory and splits it into files, calling the method on each file.
	 */
	/*public static void processDirectory(String directory, double perOverlap, double perFamily){
		//Make a directory for output
		overlapPercent = perOverlap;
		familyPercent = perFamily;
		File outputDirectory = new File(directory + "/Output");
		try{
			if(outputDirectory.mkdir())
				System.out.println("Directory Created");
			else
				System.out.println("Directory is not created");
		}
		catch(Exception e){
			e.printStackTrace();
		}

		//Get the directory and seperate it into each file
		File testFile = new File(directory);
		File[] allFiles = testFile.listFiles();
		directory = directory + "Output/";
		for (int i=0; i<allFiles.length; i++){
			if ((!(allFiles[i].isHidden())) && (!(allFiles[i].isDirectory()))){
				try {
					processFile(directory, allFiles[i].getAbsolutePath(), stripFileExtension(allFiles[i].getName()));
				}
				catch (Exception e){
					System.out.println("File not created");
				}
			}
		}
	}*/

	//Beth's updated version
	/*
	 * This method is called by the GUI
	 * It takes a directory and splits it into files, calling the method on each file.
	 */
	public static void processDirectory(String directory, double perOverlap, double perFamily, boolean graph, boolean track) throws HDCNVInputException{
		//Make a directory for output
		overlapPercent = perOverlap;
		familyPercent = perFamily;
		graphFiles = graph;
		trackFiles = track;
		File outputDirectory = new File(directory + "Output");
		//BETH... I've added options for track files and graph files.. I fixed just this one, but can you
		//check all areas you do track file stuff to see if they want them? I wasn't sure where all you
		//do it so I didn't want to do them all. I've just been testing it with track files selected
		
		try{
			if(outputDirectory.mkdir())
				System.out.println("Directory Created");
			else
				System.out.println("Directory " + directory + "Output" + " is not created");
		}
		catch(Exception e){
			e.printStackTrace();
		}
		
		
		if (trackFiles){
			File trackOutputDirectory = new File(directory + "Output/UCSC_Tracks");
			try {
				if(trackOutputDirectory.mkdir())
					System.out.println("Track Directory Created");
				else
					System.out.println("Track Directory " + directory + "Output/UCSC_Tracks" + "  is not created");
			}
			catch(Exception e){
				e.printStackTrace();
			}
		}
		

		//Get the directory and separate it into each file
		File testFile = new File(directory);
		File[] allFiles = testFile.listFiles(new HDCNVFormatFilter());
		directory = directory + "Output/";
		for (int i=0; i<allFiles.length; i++){
			if ((!(allFiles[i].isHidden())) && (!(allFiles[i].isDirectory()))){
				try {
					processFile(directory, allFiles[i].getAbsolutePath(), stripFileExtension(allFiles[i].getName()));
				}
				catch (HDCNVInputException ie){
					System.out.println("Unable to read input header from file: " + allFiles[i].getName());
					throw(ie);
				}
				catch (Exception e){
					System.out.println("File not created");
					e.printStackTrace();
				}
			}
		}



		if(trackFiles) {
			//Now have all files - Read in the individual track files and combine into one summary file
			directory = directory + "UCSC_Tracks/";
			testFile = new File(directory);
			allFiles = testFile.listFiles();

			String outputFileName = directory + "Summary_UCSCTrack.BED";
			FileWriter outputFile;
			BufferedWriter outputWriter;
			FileReader inputFile;
			BufferedReader inputReader;
			try{
				outputFile = new FileWriter(outputFileName);
				outputWriter = new BufferedWriter(outputFile);
				//only want to add the track information once.  Flag for first time through
				boolean first = true;
				for (int i=0; i<allFiles.length; i++){ 
					//if it is a Merge_UCSCTrack file (as specified in the CreateOutput writeMergeTrackFile method)  and is not a directory
					if ((allFiles[i].getName().indexOf("Merge_UCSCTrack")!=-1) && (!(allFiles[i].isDirectory()))){
						try {
							//summarize track file into one
							inputFile = new FileReader(allFiles[i]);
							inputReader = new BufferedReader(inputFile);
							
							if(!first){
								//will read over the first line in all but the first file - this is so the track name is not listed more than once
								inputReader.readLine();
							}
							//copy the file
							while(inputReader.ready()){
								outputWriter.write(inputReader.read());
							}
							
							//switch off the first flag to cause track info at top of each file to be skipped from now on.
							if(first){
								first=!first;
							}
						}
						catch (Exception e){
							System.out.println("File not created");
						}
					}
				}
				//go through again for all the samples, and add to the file
				first = true;
				for (int i=0; i<allFiles.length; i++){ 
					//if it is a Merge_UCSCTrack file (as specified in the CreateOutput writeMergeTrackFile method)  and is not a directory
					if ((allFiles[i].getName().indexOf("Samples_UCSCTrack")!=-1) && (!(allFiles[i].isDirectory()))){
						try {
							//summarize track file into one
							inputFile = new FileReader(allFiles[i]);
							inputReader = new BufferedReader(inputFile);
							
							if(!first){
								//will read over the first line in all but the first file - this is so the track name is not listed more than once
								inputReader.readLine();
							}
							//copy the file
							while(inputReader.ready()){
								outputWriter.write(inputReader.read());
							}
							
							//switch off the first flag to cause track info at top of each file to be skipped from now on.
							if(first){
								first=!first;
							}
						}
						catch (Exception e){
							System.out.println("File not created");
						}
					}
				}
				outputWriter.close();
				System.out.println("Summarizing " + allFiles.length + " Track Files");
			}
			catch (Exception e){
				e.printStackTrace();
			}	
			
		}
	}




	/*
	 * Process each file.. this means building the graph and generating the output
	 * Each file is 1 chromosome
	 * Create output creates 1 hashtable for the geno, tissue and type... that makes sense because we want to sum across chromosome?
	 */
	public static void processFile(String directory, String fileName, String fileNameWithoutExtension) throws HDCNVInputException{
		
		buildGraphFromFile(fileName);
		CreateOutput.generateOutput(overlapPercent, familyPercent, fileNameWithoutExtension, directory);
	}
	/*
	 * This method strips the file extension from the file name
	 * Code from snipplr.com
	 */
	public static String stripFileExtension(String fileName){
		File tmpFile = new File(fileName);
		tmpFile.getName();
		int whereDot = tmpFile.getName().lastIndexOf('.');
		if (0 < whereDot && whereDot <= tmpFile.getName().length() - 2 ) {
			return tmpFile.getName().substring(0, whereDot);
			//extension = filename.substring(whereDot+1);
		}    
		return "";
	}


	/*
	 * Main method
	 * Just for practice
	 */
	//	public static void main(String[] args){
	//		String directory = "/Users/jenna/Desktop/temp/";
	//		//processDirectory(directory);
	//
	//		String fileName = "/Users/jenna/Desktop/HDCNVPlayFiles/testDummy1.csv/";
	//		//parseFile(fileName);
	//		//BuildGraphFromFile(fileName, 40, 99);
	//		Set<CNV> testSet = CNVGraph.vertexSet();
	//		Iterator<CNV> it = testSet.iterator();
	//		FileWriter out = null;
	//		while (it.hasNext()){
	//			System.out.println(it.next());
	//		}
	//		File outputFile = new File("/Users/jenna/Desktop/output.txt");
	//		try{
	//			out = new FileWriter(outputFile);
	//		}
	//		catch (Exception e){
	//			e.printStackTrace();
	//		}
	//		MatrixExporter test = new MatrixExporter();
	//		test.exportAdjacencyMatrix(out, CNVGraph);
	//		System.out.println("***************************************");
	//		Set<DefaultEdge> edges = CNVGraph.edgeSet();
	//		Iterator<DefaultEdge> itE = edges.iterator();
	//		while (itE.hasNext()){
	//			System.out.println(itE.next());
	//		}
	//		System.out.println("***************************************");
	//
	//		System.out.println("***************************************");
	//
	//		//This will print out the families.
	//		Set<CNV> vertexSet = CNVGraph.vertexSet();
	//		Iterator<CNV> vertexIt = vertexSet.iterator();
	//		while (vertexIt.hasNext()){
	//			System.out.println(vertexIt.next().getFamilyString());
	//		}
	//		System.out.println("***************************************");
	//
	//		//This will print out everything with 40% overlap... Awesome possum.
	//		ConnectivityInspector<CNV, DefaultEdge> ins = new ConnectivityInspector<CNV, DefaultEdge>(CNVGraph);
	//		List<Set<CNV>> result = ins.connectedSets();
	//		Iterator<Set<CNV>> it2 = result.iterator();
	//		while (it2.hasNext()){
	//			Set<CNV> temp = it2.next();
	//			Iterator<CNV> it3 = temp.iterator();
	//			while (it3.hasNext()){
	//				System.out.print(it3.next().getID() + ", ");
	//			}
	//			System.out.println();
	//		}
	//
	//	}

}
