/*
 * CNV Objects make up the nodes of the graph, and store all the details on each CNV event
 * Author: Jenna Butler
 */

//package jennabutler.hdcnv;
import java.util.LinkedList;
import java.util.Iterator;

public class CNV {

	int ID;
	String chromo;
	int start;
	int end;
	int sampleID;
	String CNType;
	int length;
	int markers;
	double pValue;
	int knownSNPs;
	String geno;
	String tissue;
	boolean emptyFamily;
	String extra;
	//Perhaps have a linked list of family members?
	LinkedList<Integer> family;
	
	public CNV(){
		ID = -1;
		emptyFamily = true;
		//this.family = new LinkedList<Integer>();
	}
	
	//Minimum information
	/*
	 * Constructor for CNV with minimal information
	 */
	public CNV(int ID, String chromo, int start, int end, int sample, String CNType){
		this.ID = ID;
		this.chromo = chromo;
		this.start = start;
		this.end = end;
		this.CNType = CNType;
		this.sampleID = -1;
	}
	
	public CNV(int ID, String chromo, int start, int end, int sample, String CNType, String extra){
		this.ID = ID;
		this.chromo = chromo;
		this.start = start;
		this.end = end;
		this.CNType = CNType;
		this.sampleID = -1;
		this.extra = extra;
	}
	
	public CNV(int ID, String chromo, int start, int end, int sampleID, String CNType, int length, int markers, double pValue, int knownSNPs){
		this.ID = ID;
		this.chromo = chromo;
		this.start = start;
		this.end = end;
		this.sampleID = sampleID;
		this.CNType = CNType;
		this.length = length;
		this.markers = markers;
		this.pValue = pValue;
		this.knownSNPs = knownSNPs;
		//this.family = new LinkedList<Integer>();
		//emptyFamily = true;
	}
	
	public void initalizeFamily(){
		this.family = new LinkedList<Integer>();
	}

	public double getpValue() {
		return pValue;
	}

	public void setpValue(double pValue) {
		this.pValue = pValue;
	}

	public int getID() {
		return ID;
	}

	public void setID(int iD) {
		ID = iD;
	}

	public String getChromo() {
		return chromo;
	}

	public void setChromo(String chromo) {
		this.chromo = chromo;
	}

	public int getStart() {
		return start;
	}

	public void setStart(int start) {
		this.start = start;
	}

	public int getEnd() {
		return end;
	}

	public void setEnd(int end) {
		this.end = end;
	}

	public int getSampleID() {
		return sampleID;
	}

	public void setSampleID(int sampleID) {
		this.sampleID = sampleID;
	}

	public String getCNType() {
		return CNType;
	}

	public void setCNType(String cNType) {
		CNType = cNType;
	}

	public int getLength() {
		return length;
	}

	public void setLength(int length) {
		this.length = length;
	}

	public int getMarkers() {
		return markers;
	}

	public void setMarkers(int markers) {
		this.markers = markers;
	}

	public int getKnownSNPs() {
		return knownSNPs;
	}

	public void setKnownSNPs(int knownSNPs) {
		this.knownSNPs = knownSNPs;
	}
	
	public String getGeno(){
		return geno;
	}
	
	public String getTissue(){
		return tissue;
	}
	
	public void setExtra(String extra){
		this.extra = extra;
	}
	
	public String getExtra(){
		return extra;
	}
	
	//Add a family member to the list
	public void addFamilyMember(Integer member){
		if (emptyFamily) {
			family.add(ID);
			emptyFamily = false;
		}
		family.add(member);
	}
	
	//Return an iterator over the family
	public Iterator<Integer> getFamily(){
		return family.iterator();
	}
	
	//Print out the family members to this CNV
	public String getFamilyString(){
		String familyString = "";
		if (family != null){
			Iterator<Integer> it = family.iterator();
			while (it.hasNext()){
				familyString = familyString + it.next() + " ";
			}
			familyString = familyString.substring(0, familyString.length()-1);
		}
		return familyString;
	}
	
	public void setTissueAndGeno(){
		Iterator<SampleInfo> allSamples = FormatInput.samples.iterator();
		SampleInfo temp = null;
		boolean foundSample = false;
		while (allSamples.hasNext()){
			temp = allSamples.next();
			if ((int)(temp.getSampleNum()) == sampleID){
				foundSample = true;
				geno = temp.getGeno();
				tissue = temp.getTissue();
				break;
			}
		}
		if (foundSample == false){
			geno = "unknown";
			tissue = "unknown";
		}
	}
	
	public String trackString(String colorString){
		return (chromo + "\t"+
				start+ "\t"+
				end+ "\t"+
				ID+"\t"+
				0+"\t"+
				"+"+"\t"+
				start+"\t"+
				start+"\t"+
				colorString +"\t"+
				0+"\t"+
				"\t"+
				"CNV"+"\t" +
				"<b>ID in input file: </b>"+ ID + "\n"
				);
	}
	
	public String trackString(){
		return trackString("0");
	}
	
	public String toString(){
		String data = ("SNP ID: " + ID);
		data = data + (" On chromo: " + chromo + "\n");
		data = data + ("Starts at: " + start + " and ends at: " + end + "\n");
		data = data + ("From sample: "+ sampleID + " and is a: "+CNType + "\n");
		data = data + ("The length is: " + length + " and has "+ markers + " markers. \n");
		data = data + ("It has a pvalue of "+ pValue + " and " + knownSNPs + " known snps.");
		return data;
	}
	
}
