/*
 * This class handles making all of the output files
 * Authors: Jenna Butler and Beth Locke
 */

//package jennabutler.hdcnv;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.*;
import java.util.regex.Pattern;


import org.jgrapht.alg.*;
import org.jgrapht.*;
import org.jgrapht.graph.*;

import java.io.*;
import org.jgrapht.ext.*;


public class CreateOutput {

	private static Hashtable<String, Integer> genoHash;
	private static Hashtable<String, Integer> tissueHash;
	private static Hashtable<String, Integer> typeHash;
	private static FileWriter summaryOutputFile;
	private static BufferedWriter summaryOutputWriter;
	
	//Colours used for generating Tracks
	private static final String SAMPLE_AMP_TRACK_COLOUR = "50,200,50";
	private static final String SAMPLE_DEL_TRACK_COLOUR = "200,50,50";
	private static final String HDCNV_TRACK_COLOUR = "0,60,120";
	
	private static final String BASE_URL = "http://daleylab.org/lab/?page_id=125";


	public static void generateOutput(double overlapPercent, double familyPercent, String chromo, String directory){
		//Create files for this chromo
		genoHash = new Hashtable<String, Integer>();
		tissueHash = new Hashtable<String, Integer>();
		typeHash = new Hashtable<String, Integer>();
		//String detailedMergeString = detailedMergeString(overlapPercent, familyPercent) + "\n\n";
		String detailedMergeString = cliqueFinder(overlapPercent, familyPercent) + "\n\n";
		detailedMergeString = detailedMergeString + "Genotype summary: \n" + summarizeGenos() + "\n";
		detailedMergeString = detailedMergeString + "Tissue summary: \n" + summarizeTissues() + "\n";
		detailedMergeString = detailedMergeString + "CNV Type summary: \n" + summarizeTypes() + "\n";
		//Write the summary file
		writeSummaryFile(detailedMergeString, chromo, directory);

		//Write the normal output file
		String regularOutputString = vertexList(overlapPercent, familyPercent);
		writeOutputFile(regularOutputString, chromo, directory, "output.csv");
		//String summaryString = cliqueFinder(overlapPercent, familyPercent);

		//Format Gephi File
		GephiFormatting(chromo, directory);

		//check global variable to see whether or not to generate track files
		if(BuildGraph.trackFiles){
			//Create Track String for output

			int chrInt = getChrInteger(chromo);
			
			if(chrInt==0){
				System.out.println ("Could not create track file for: " + chromo);
			}
			
			else {
				
				String sampleTrackString = sampleTrackString(chrInt);
				writeSampleTrackFile(sampleTrackString, chromo, directory+"UCSC_Tracks/");
				
				String trackString = trackString(overlapPercent, familyPercent, chrInt);
				writeMergeTrackFile(trackString, chromo, directory+"UCSC_Tracks/");
			}
		}
	}


	/*
	 * Write the regular output file
	 */
	private static void writeOutputFile(String output, String chromo, String directory, String fileName){
		String outputFileName = directory + chromo+ fileName;
		FileWriter outputFile;
		BufferedWriter outputWriter;
		try{
			outputFile = new FileWriter(outputFileName);
			outputWriter = new BufferedWriter(outputFile);
			outputWriter.write(output);
			outputWriter.close();
		}
		catch (Exception e){
			e.printStackTrace();
		}	
	}


	/*
	 * Write a Track file for UCSC Genome Browser
	 * @author: Beth
	 */
	private static void writeMergeTrackFile(String output, String chromo, String directory){
		String outputFileName = directory +chromo+"Merge_UCSCTrack.BED";
		FileWriter outputFile;
		BufferedWriter outputWriter;
		try{
			//open a file writer in append mode 
			outputFile = new FileWriter(outputFileName);
			outputWriter = new BufferedWriter(outputFile);
			outputWriter.write(output);
			outputWriter.close();
		}
		catch (Exception e){
			e.printStackTrace();
		}	
	}

	/*
	 * Write a Track file for UCSC Genome Browser
	 * @author: Beth
	 */
	private static void writeSampleTrackFile(String output, String chromo, String directory){
		String outputFileName = directory +chromo+"Samples_UCSCTrack.BED";
		FileWriter outputFile;
		BufferedWriter outputWriter;
		try{
			//open a file writer in append mode 
			outputFile = new FileWriter(outputFileName);
			outputWriter = new BufferedWriter(outputFile);
			outputWriter.write(output);
			outputWriter.close();
		}
		catch (Exception e){
			e.printStackTrace();
		}	
	}
	/*
	 * Method to make a string of merges in the regular output file
	 */
	private static String getMergeString(CNV current){
		int idNum = current.getID();
		String merges = "";
		int count = 0;
		while (count < BuildGraph.numOfNodes){
			if (BuildGraph.adjacencyMatrix[count][idNum] == 1){
				merges = merges + Integer.toString(count) + " ";
			}
			count++;
		}
		return merges;
	}

	/*
	 * Write the compressed output file
	 */
	private static void writeSummaryFile(String output, String chromo, String directory){
		String summaryOutputFileName = directory + chromo+"SummaryOutput.csv";
		FileWriter summaryOutputFile;
		BufferedWriter summaryOutputWriter;
		try{
			summaryOutputFile = new FileWriter(summaryOutputFileName);
			summaryOutputWriter = new BufferedWriter(summaryOutputFile);
			summaryOutputWriter.write(output);
			summaryOutputWriter.close();
		}
		catch (Exception e){
			e.printStackTrace();
		}	
	}

	/*
	 * Trying the BronKerboschCliqueFinder(Graph<V,E> graph) 
	 */
	private static String cliqueFinder(double overlapPercent, double familyPercent){
		BronKerboschCliqueFinder<CNV, DefaultEdge> cliques = new BronKerboschCliqueFinder<CNV, DefaultEdge>(BuildGraph.CNVGraph);
		Collection<Set<CNV>> cliqueSet  = cliques.getAllMaximalCliques();
		Iterator<Set<CNV>> cliqueIt = cliqueSet.iterator();
		//This method should go through the graph and print out all the merged regions
		int count = 0;
		CNV current = null;
		String mergeString = "Paramters:, Percent Overlap: " + overlapPercent + ", Family Overlap: " + familyPercent + "\n";
		mergeString = mergeString + "MergeID, Samples in Merge, Start of Merge, End of Merge, Length of Merge, Geno, Tissue, Type, Sample IDs \n";
		//Store all of the Tissues, Types and Genos in a hash table that is consistent for each chromosome..

		while (cliqueIt.hasNext()){
			Set<CNV> currentSet = cliqueIt.next();
			if (currentSet.size() > 1){
				String testMergeSetString = mergeSet(currentSet, count);
				mergeString = mergeString + testMergeSetString + "\n";
				//genoString will be what type of genos is in the particluar merge
				hashGenos(currentSet);
				hashTissue(currentSet);
				hashType(currentSet);
				count++;
			}
		}
		return mergeString;
		//while (cliqueIt.hasNext()){
		//System.out.println("************************");
		//System.out.println(cliqueIt.next());
		//}
	}

	/*
	 * Method to return a string with the merges
	 * It will find the connected components to count merge
	 */
	public static String findMergeString(double overlapPercent, double familyPercent){
		//This method should go through the graph and print out all the merged regions
		ConnectivityInspector<CNV, DefaultEdge> ins = new ConnectivityInspector<CNV, DefaultEdge>(BuildGraph.CNVGraph);
		List<Set<CNV>> result = ins.connectedSets();
		Iterator<Set<CNV>> it2 = result.iterator();
		String mergeString = "Paramters:, Percent Overlap: " + overlapPercent + ", Family Overlap: " + familyPercent + "\n";
		while (it2.hasNext()){
			Set<CNV> temp = it2.next();
			Iterator<CNV> it3 = temp.iterator();
			while (it3.hasNext()){
				mergeString = mergeString+(it3.next().getID() + ", ");
				//System.out.print(it3.next().getID() + ", ");
			}
			mergeString = mergeString.substring(0, mergeString.length()-2);
			mergeString = mergeString + "\n";
			//System.out.println();
		}
		return mergeString;
	}


	/*
	 * Method to print out all details of a merged string
	 */
	public static String detailedMergeString(double overlapPercent, double familyPercent){
		//This method should go through the graph and print out all the merged regions
		int count = 0;
		ConnectivityInspector<CNV, DefaultEdge> connectInspect = new ConnectivityInspector<CNV, DefaultEdge>(BuildGraph.CNVGraph);
		List<Set<CNV>> result = connectInspect.connectedSets();
		Iterator<Set<CNV>> connectedIt = result.iterator();
		CNV current = null;
		String mergeString = "Paramters:, Percent Overlap: " + overlapPercent + ", Family Overlap: " + familyPercent + "\n";
		mergeString = mergeString + "MergeID, Samples in Merge, Start of Merge, End of Merge, Length of Merge, Geno, Tissue, Type, Sample IDs \n";
		//Store all of the Tissues, Types and Genos in a hash table that is consistent for each chromosome..

		while (connectedIt.hasNext()){
			Set<CNV> currentSet = connectedIt.next();
			if (currentSet.size() > 1){
				String testMergeSetString = mergeSet(currentSet, count);
				mergeString = mergeString + testMergeSetString + "\n";
				//genoString will be what type of genos is in the particluar merge
				hashGenos(currentSet);
				hashTissue(currentSet);
				hashType(currentSet);
				count++;
			}
		}
		return mergeString;
	}


	/*
	 * @author Beth
	 * Create a string for the track
	 */
	public static String trackString( double overlapPercent, double familyPercent, int chromo){

		BronKerboschCliqueFinder<CNV, DefaultEdge> cliques = new BronKerboschCliqueFinder<CNV, DefaultEdge>(BuildGraph.CNVGraph);
		Collection<Set<CNV>> cliqueSet  = cliques.getAllMaximalCliques();
		Iterator<Set<CNV>> cliqueIt = cliqueSet.iterator();
		//This method should go through the graph and print out all the merged regions
		int count = 0;

		String trackString= "track name=HD-CNV type=bedDetail description=\"Detected CNV Hotspots\" visibility=2 color="+HDCNV_TRACK_COLOUR+" url=\""+ BASE_URL +"#$$\"\n";
		//Store all of the Tissues, Types and Genos in a hash table that is consistent for each chromosome..

		while (cliqueIt.hasNext()){
			Set<CNV> currentSet = cliqueIt.next();
			if (currentSet.size() > 1){
				String testMergeSetString = mergeTrackSet(currentSet, count, chromo, overlapPercent, familyPercent);
				trackString = trackString + testMergeSetString + "\n";
				//genoString will be what type of genos is in the particluar merge
				hashGenos(currentSet);
				hashTissue(currentSet);
				hashType(currentSet);
				count++;
			}
		}

		//insert store all information 
		return trackString;
	}

	/*
	 * @author Beth
	 * Create a string for the track
	 */
	public static String sampleTrackString( int chromo){
		
	
		String trackString= "track name=HD-CNV-SAMPLES type=bedDetail description=\"CNVs in Samples\" visibility=2 itemRgb=\"On\" url=\""+ BASE_URL +"#$$\"\n";
		//Store all of the Tissues, Types and Genos in a hash table that is consistent for each chromosome..

		//iterate through the graph
		Set<CNV> vertecies = BuildGraph.CNVGraph.vertexSet();
		Iterator<CNV> it = vertecies.iterator();
		CNV current;
		
		while (it.hasNext()){
			current = it.next();
			if(current.getCNType().toLowerCase().contains("amp")){
				trackString += current.trackString(SAMPLE_AMP_TRACK_COLOUR);
			}
			else{
				trackString += current.trackString(SAMPLE_DEL_TRACK_COLOUR);
			}
		}

		//insert store all information 
		return trackString;
	}

	
	/*
	 * Method to merge a set of merged CNVS for output
	 */
	private static String mergeSet(Set<CNV> toMerge, int mergedID){
		final int ARRAY_SIZE = 10000;
		Iterator<CNV> cnvIt = toMerge.iterator();
		java.text.DecimalFormat leadingZerosFormat = new java.text.DecimalFormat("#000.###");
		String leadingZeroMergeID = leadingZerosFormat.format(mergedID);
		String mergeSetString = "MergeID" + leadingZeroMergeID + ",";
		CNV current = null;
		int[] startingPositions = new int[ARRAY_SIZE];
		int[] endingPositions = new int[ARRAY_SIZE];
		int[] idArray = new int[ARRAY_SIZE];
		int[] sampleIdArray = new int[ARRAY_SIZE];
		String[] tissueArray = new String[ARRAY_SIZE];
		String[] typeArray = new String[ARRAY_SIZE];
		String[] genoArray = new String[ARRAY_SIZE];
		int count = 0;
		while (cnvIt.hasNext()){
			current = cnvIt.next();
			sampleIdArray[count] = current.getSampleID();
			idArray[count] = current.getID();
			startingPositions[count] = current.getStart();
			endingPositions[count] = current.getEnd();
			tissueArray[count] = current.getTissue();
			typeArray[count] = current.getCNType();
			genoArray[count] = current.getGeno();
			count++;
		}
		String IDsInMerge = idArrayToString(idArray, count);
		String sampleIDs = sampleArrayToString(sampleIdArray, count);
		int start = getSmallest(startingPositions, count);
		int end = getLargest(endingPositions, count);
		int length = end - start;
		String tissueType = CompressTypes(tissueArray);
		String genoType = CompressTypes(genoArray);
		String typeType = CompressTypes(typeArray);

		//Create the big merge string
		mergeSetString = mergeSetString + IDsInMerge + "," + start + "," + end + "," + length + "," + genoType + "," + tissueType + "," + typeType + "," + sampleIDs;
		
		return mergeSetString;
	}


	/* 
	 * Same as above method, but creats a merge string in track format
	 * @author Beth
	 */
	private static String mergeTrackSet(Set<CNV> toMerge, int mergedID, int chromo, double overlapPercent, double familyPercent){
		final int ARRAY_SIZE = 10000;
		Iterator<CNV> cnvIt = toMerge.iterator();
		java.text.DecimalFormat leadingZerosFormat = new java.text.DecimalFormat("#000.###");
		String leadingZeroMergeID = leadingZerosFormat.format(mergedID);

		CNV current = null;
		int[] startingPositions = new int[ARRAY_SIZE];
		int[] endingPositions = new int[ARRAY_SIZE];
		int[] idArray = new int[ARRAY_SIZE];
		String[] tissueArray = new String[ARRAY_SIZE];
		String[] typeArray = new String[ARRAY_SIZE];
		String[] genoArray = new String[ARRAY_SIZE];
		int count = 0;
		while (cnvIt.hasNext()){
			current = cnvIt.next();
			idArray[count] = current.getID();
			startingPositions[count] = current.getStart();
			endingPositions[count] = current.getEnd();
			tissueArray[count] = current.getTissue();
			typeArray[count] = current.getCNType();
			genoArray[count] = current.getGeno();
			count++;
		}
		String IDsInMerge = idArrayToString(idArray, count);
		int start = getSmallest(startingPositions, count);
		int end = getLargest(endingPositions, count);

		String tissueType = CompressTypes(tissueArray);
		String genoType = CompressTypes(genoArray);
		String typeType = CompressTypes(typeArray);

		//create line in track file:
		//chromo <\t> start <\t> end <\t> family_name <\t> score <\n>

		String mergeSetString = "chr" + chromo + "\t" + 
				start + "\t" + 
				end + "\t" +
				"MergeID" + leadingZeroMergeID + "\t" + 
				count + "\t" +
				"+\t" +
				start + "\t" +  //thickness start - set to merge start
				start + "\t" +  //thickness end - set to merge start also SO NONE OF THE TRACK IS THICK
				"0" + "\t" +   //block count - we have no blocks
				"0" + "\t" +   //block sizes - seem to need a zero here. This line and the next are hardcoded magic for BED files
				"\t" +			//block starts - seem to need nothing here.  Again BED magic.  
				"Merge" + "\t"+		//This is used with the url listed in the track line of the file to index to a certain anchor in the HDCNV page
				//details line for the track detail page
				"<br><br><b>CNVs In Merge: </b>" +  IDsInMerge +
				"<br><b>Tissue Type: </b>" + tissueType +
				"<br><b>Genotype:</b> " + genoType + 
				"<br><b>Type:</b> " + typeType +
				"<h3>Program Details</h3> <b>Parameters <br> &nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbspPercent Overlap:</b> " + overlapPercent +
				"<br> <b>&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbspFamily Overlap:</b> " + 	familyPercent;  
		return mergeSetString;
	}


	/*
	 * Helper method to find the tissue type for a merge
	 */
	private static String CompressTypes(String[] bigString){
		int count = 0;
		int count2 = 0;
		String temp1 = "";
		String temp2 = "";
		boolean different = false;
		while (bigString[count] != null) {
			temp1 = bigString[count];
			while (bigString[count2] != null){
				temp2 = bigString[count2];
				if (!temp1.equals(temp2)){
					//if (temp1 != bigString[count2]){
					different = true;
					temp1 = "mix";
					break;
				}
				count2++;
			}
			if (different)
				break;
			count++;
		}
		return temp1;
	}

	/*
	 * Helper method to squish an array into a string
	 */
	private static String idArrayToString(int[] idArray, int count){
		String ids = "";
		int i=0;
		while (i<count){
			ids = ids + idArray[i] + " ";
			i++;
		}
		return ids;
	}

	/*
	 * Helper method to squish an array into a string
	 */
	private static String sampleArrayToString(int[] idArray, int count){
		String ids = "";
		int i=0;
		while (i < count){
			if(i+1 == count){
				ids = ids + idArray[i];
			}
			else {
			ids = ids + idArray[i] + ", ";
			}
			i++;
		}
		return ids;
	}
	/*
	 * helper method to find smallest starting point for merge
	 */
	private static int getSmallest(int[] positions, int count){
		int currentSmallest = positions[0];
		int i=1;
		while (i < count){
			if (positions[i] < currentSmallest){
				currentSmallest = positions[i];
			}
			i++;
		}
		return currentSmallest;
	}
	/*
	 * Helper method to find the ending position
	 */
	private static int getLargest(int[] positions, int count){
		int currentLargest = positions[0];
		int i=1;
		while (i < count){
			if (positions[i] > currentLargest){
				currentLargest = positions[i];
			}
			i++;
		}
		return currentLargest;
	}

	/*
	 * Helper method to hash the genos
	 * This uses an iterator from a set, that is also passed to another method and iterated on.. is that okay?? MARK TEST
	 */
	private static void hashGenos(Set<CNV> cnvSet){
		Iterator<CNV> cnvIt = cnvSet.iterator();
		CNV current = null;
		String currentGeno;
		Integer intgr;
		while (cnvIt.hasNext()){
			current = cnvIt.next();
			currentGeno = current.getGeno();
			intgr = (Integer) (genoHash.get(currentGeno));
			if (intgr==null){
				genoHash.put(currentGeno, new Integer(1));
			}
			else {
				genoHash.put(currentGeno,  new Integer(intgr.intValue() + 1));
			}
		}
	}


	/*
	 * Helper method to hash the genos
	 * This uses an iterator from a set, that is also passed to another method and iterated on.. is that okay?? MARK TEST
	 */
	private static void hashTissue(Set<CNV> cnvSet){
		Iterator<CNV> cnvIt = cnvSet.iterator();
		CNV current = null;
		String currentTissue;
		Integer intgr;
		while (cnvIt.hasNext()){
			current = cnvIt.next();
			currentTissue = current.getTissue();
			intgr = (Integer) (tissueHash.get(currentTissue));
			if (intgr==null){
				tissueHash.put(currentTissue, new Integer(1));
			}
			else {
				tissueHash.put(currentTissue,  new Integer(intgr.intValue() + 1));
			}
		}
	}


	/*
	 * Helper method to hash the genos
	 * This uses an iterator from a set, that is also passed to another method and iterated on.. is that okay?? MARK TEST
	 */
	private static void hashType(Set<CNV> cnvSet){
		Iterator<CNV> cnvIt = cnvSet.iterator();
		CNV current = null;
		String currentType;
		Integer intgr;
		while (cnvIt.hasNext()){
			current = cnvIt.next();
			currentType = current.getCNType();
			intgr = (Integer) (typeHash.get(currentType));
			if (intgr==null){
				typeHash.put(currentType, new Integer(1));
			}
			else {
				typeHash.put(currentType,  new Integer(intgr.intValue() + 1));
			}
		}
	}

	/*
	 * Method to print out each vertex and the corresponding information...
	 */
	public static String vertexList(double overlapPercent, double familyPercent){
		//Print the adjacency matrix
		//System.out.println(printAdjacencyMatrix());
		String vString = "Paramters:, Percent Overlap: " + overlapPercent + ", Family Overlap: " + familyPercent + "\n";
		vString = vString + "ID, Start, End, Merges With, Family With, Tissue, Genotype, CNV Type, Sample ID";
		//If there was additional columns in the file, we want to add them back in
		if (FormatInput.extraInfo){
			vString = vString + FormatInput.header + "\n";
		}
		else {
			vString = vString + "\n";
		}
		Iterator<CNV> vertexList = BuildGraph.CNVGraph.vertexSet().iterator();
		CNV current = null;
		int count = 0;
		String family = "";
		while (vertexList.hasNext()){
			family = "";
			//reset count to 0 for adjacency matrix
			count = 0;
			//get current CNV from vertex list
			current = vertexList.next();
			//Enter basic information in output string
			vString = vString + current.getID() + "," + current.getStart() + "," + current.getEnd() + "," + current.getID() +" ";
			//Loop through adjacency matrix to see what it merges with
			while (count < BuildGraph.numOfNodes){
				//If there is a one for the current node and count, than it matches with the count node.. add to the list
				//Have to add count plus 1 because CNV1 is at slot 0, CNV2 is at slot 1... etc.
				if (BuildGraph.adjacencyMatrix[current.getID()-1][count] == 1){
					vString = vString + (count+1) + " ";
				}
				count ++;
			}
			family = current.getFamilyString();
			if (family.equals("")){
				family = "No family";
			}
			if (FormatInput.extraInfo){
				vString = vString + "," + family + "," + current.getTissue() + "," + current.getGeno() + "," + current.getCNType() + "," + current.getSampleID() + current.getExtra() + "\n";
			}
			else {
				vString = vString + "," + family + "," + current.getTissue() + "," + current.getGeno() + "," + current.getCNType() + "," + current.getSampleID() + "\n";

			}
			//Add the families
			//Now calculate families
		}
		return vString;
	}

	/*
	 * Method to count how many of each tissue, geno and type events are in a whole chromosome
	 */
	private static String summarizeGenos(){
		String type = "";
		String sumGenos = "";
		for (Enumeration e = genoHash.keys(); e.hasMoreElements();)
		{
			type = (String)(e.nextElement());
			System.out.println("There are: " +genoHash.get(type) + " " + type);
			sumGenos = sumGenos + type + ": " + genoHash.get(type) + "\n";
		}
		return sumGenos;
	}

	/*
	 * Method to count how many of each tissue, geno and type events are in a whole chromosome
	 */
	private static String summarizeTissues(){
		String type = "";
		String sumTissues = "";
		for (Enumeration e = tissueHash.keys(); e.hasMoreElements();)
		{
			type = (String)(e.nextElement());
			sumTissues = sumTissues + type + ": " + tissueHash.get(type) + "\n";
		}
		return sumTissues;
	}

	/*
	 * Method to count how many of each tissue, geno and type events are in a whole chromosome
	 */
	private static String summarizeTypes(){
		String type = "";
		String sumTypes = "";
		for (Enumeration e = typeHash.keys(); e.hasMoreElements();)
		{
			type = (String)(e.nextElement());
			sumTypes = sumTypes + type + ": " + typeHash.get(type) + "\n";
		}
		return sumTypes;
	}


	/*
	 * Method to print contents of adjacency matrix
	 */
	public static String printAdjacencyMatrix(){
		System.out.println(BuildGraph.numOfNodes);
		String adMatrix = "";
		for (int i=0; i<BuildGraph.numOfNodes;i++){
			for (int j=0; j<BuildGraph.numOfNodes;j++){
				adMatrix = adMatrix + BuildGraph.adjacencyMatrix[i][j] + " ";
			}
			adMatrix = adMatrix + "\n";
		}
		return adMatrix;
	}

	/*
	 * @Author Beth
	 * Get the int number from the chromosome input string.  
	 * Ex. "run2Chr10" should return 10.
	 */
	public static int getChrInteger(String input){
		Scanner scan = new Scanner(input);

		int toReturn=0;
		try{
			//Case insensitive
			toReturn = Integer.parseInt(scan.findInLine(Pattern.compile("[Cc][Hh][Rr]\\d+")).substring(3));
		}
		catch(Exception e){
			System.out.println("Filename must contain chr## (case insensitive) to indicate Chromosome for track file");
		}
		return toReturn;
	}

	/*
	 * @Author Jenna
	 * Method to format output file for Gephi
	 */
	public static void GephiFormatting(String chromo, String directory){
		//String output ="";
		StringBuffer output = new StringBuffer();
		int nodeNumber = 0;
		for (int i=0; i<BuildGraph.numOfNodes; i++){
			nodeNumber = i + 1;
			//output = output + ";" + "Node" + nodeNumber;
			output.append(";ID");
			output.append(nodeNumber);
		}
		//output = output + "\n";
		output.append("\n");
		for (int i=0; i<BuildGraph.numOfNodes; i++){
			nodeNumber = i + 1;
			//output = output + "Node" + nodeNumber;
			output.append("ID");
			output.append(nodeNumber);
			for (int j=0; j<BuildGraph.numOfNodes; j++){
				//output = output + ";" + BuildGraph.adjacencyMatrix[i][j];
				output.append(";");
				output.append(BuildGraph.adjacencyMatrix[i][j]);
			}
			//output = output + "\n";
			output.append("\n");
		}
		writeOutputFile(output.toString(), chromo, directory, "Graph.csv");

	}


}
