/*
 * FormatInput Class
 * Author: Jenna Butler
 * This class takes in each file and checks the headers, pulls out sample data, and generally parses any input given 
 */

//package jennabutler.hdcnv;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.StringTokenizer;
import java.util.LinkedList;

import javax.swing.JOptionPane;
public class FormatInput {

	public static String header = "";
	public static boolean extraInfo = false;
	public static LinkedList<SampleInfo> samples = new LinkedList<SampleInfo>();

	//Jenna
	/*
	 * The idea here is to parse the file and make the graph nodes
	 * This shoudl be called instead of the current parseFile method in the buildgraph class
	 * It grabs all the required fields, then just sticks everything else in an "extra" field
	 */
	
	public static void parseFile(String fileName) throws HDCNVInputException{
		BufferedReader inFileReader = null; 
		String line = null;
		//Open the file and read the first line
		try {
			inFileReader = new BufferedReader(new FileReader(fileName));
			//line = inFileReader.readLine();
			line = inFileReader.readLine(); //first line is headers so skip that line
			//The first line has the headers.. .lets first make sure it has the right headers
		}
		catch(Exception e){
			System.out.println("Error :" + e.getMessage());
		}
	
		//CNV[] cnvs = new CNV[60];
		try {
			FormatInput.checkHeader(line);
		}
		catch (HDCNVInputException e){
			throw(e);
		}

		//once done parsing the header, get the next line
		try{
		line = inFileReader.readLine();
		}
		catch (Exception e){
			System.out.println("Error: " + e.getMessage());
		}
		//loop through the file and create the CNV objects
		while (line != null){
			if (line.charAt(0) == ','){
				break;
				//BETH OR MARK: How can I make this throw some info to the screen without terminating or propagating back to the gui?
				//throw new HDCNVInputException("A blank line was found in a file. Processing of that file was terminated. If the blank line was not at the end of the file, some data may be lost.");
			}
			StringTokenizer st = new StringTokenizer(line, ",");
			CNV temp = new CNV();
			String next = st.nextToken();
			//order of the file should be ID, chromo, start, end, sample, cntype, length, makers, pvalue, known snps
			temp.setID(Integer.parseInt(next));
			temp.setChromo(st.nextToken());
			temp.setStart(Integer.parseInt(st.nextToken()));
			temp.setEnd(Integer.parseInt(st.nextToken()));
			temp.setSampleID(Integer.parseInt(st.nextToken()));
			temp.setCNType(st.nextToken());
			int length = temp.getEnd() - temp.getStart();
			temp.setLength(length);
			String extra = "";
			for (int i=0; i<HDCNV_GUI.extraColumns; i++){
				extra = extra + ", " + st.nextToken();
			}
			temp.setExtra(extra);
			//temp.setLength(Integer.parseInt(st.nextToken()));
			//temp.setMarkers(Integer.parseInt(st.nextToken()));
			//temp.setpValue(Double.parseDouble(st.nextToken()));
			//temp.setKnownSNPs(Integer.parseInt(st.nextToken()));
			temp.setTissueAndGeno();
			BuildGraph.CNVGraph.addVertex(temp);
			BuildGraph.numOfNodes++;
			//}
			try {
				line = inFileReader.readLine();
			}
			catch (Exception e){
				System.out.println("Error: " + e.getMessage());
			}
		}
		try{
			inFileReader.close();
		}
		catch (Exception e){
			e.printStackTrace();
		}
	}
	
	//id, chromo, start, end, sample, type
	//Returns the header for use elsewhere
	/*
	 * Method looks at header and checks if all the necessary columns are present
	 * Throws an exception if not
	 * @param The first line of the file being processed
	 * @return A string with any "extra" header info (columns other than required 6)
	 */
	private static String checkHeader(String line) throws HDCNVInputException{
		StringTokenizer st1 = new StringTokenizer(line, ",");
		String token = st1.nextToken();
		String errorMessage = "";
		if (!token.equals("ID") && !(token.equals("id")) && !(token.equals("Id"))){
			errorMessage = errorMessage + "Error in column 1. ID header not detected. (File may be of incorrect format: Ensure no other .csv files are present in working directory)\n";
			throw new HDCNVInputException(errorMessage);
		}
		token = st1.nextToken();
		if ((!(token.equalsIgnoreCase("Chr"))) && (!(token.equalsIgnoreCase("Chromosome"))) && (!(token.equalsIgnoreCase("Chromo")))){
			errorMessage = errorMessage + "Error in column 2. Chromosome header not detected. \n";
			throw new HDCNVInputException(errorMessage);
		}
		token = st1.nextToken();
		if (!(token.equalsIgnoreCase("start"))){
			errorMessage = errorMessage + "Error in column 3. Start header not detected. \n";
			throw new HDCNVInputException(errorMessage);
		}		
		token = st1.nextToken();
		if (!(token.equalsIgnoreCase("end"))){
			errorMessage = errorMessage + "Error in column 4. End header not detected. \n";
			throw new HDCNVInputException(errorMessage);
		}
		token = st1.nextToken();
		if (!(token.equalsIgnoreCase("sample"))){
			errorMessage = errorMessage + "Error in column 5. Sample header not detected. \n";
			throw new HDCNVInputException(errorMessage);
		}
		token = st1.nextToken();
		if (!(token.equalsIgnoreCase("type"))){
			errorMessage = errorMessage + "Error in column 6. Type header not detected. \n";
			throw new HDCNVInputException(errorMessage);
		}
		if (!(errorMessage.equals(""))){
			throw new HDCNVInputException(errorMessage);
		}
		if (HDCNV_GUI.extraColumns != 0){
			extraInfo = true;
			for (int i=0; i<HDCNV_GUI.extraColumns; i++){
				header = header + ", " + st1.nextToken();
			}
		}
		return header;
		//return errorMessage;
	}
	
	//Go through the samples provided and make a list of possible samples
	/*
	 * Method to analyze the sample data provided from the GUI
	 * @param String of sample data, line by line per sample, from GUI
	 * 
	 */
	public static void createSampleList(String sampleData){
		StringTokenizer st1 = new StringTokenizer(sampleData, "\n");
		//String sampleLine = st1.nextToken();
		SampleInfo temp = null;
		for (int i=0; i<HDCNV_GUI.numSamples; i++){
			String sampleLine = st1.nextToken();
			if (sampleLine != null){
				temp = parseSampleLine(sampleLine);
				samples.add(temp);
			}
			//sampleLine = st1.nextToken();
		}
	}
	
	/*
	 * Method to break each line of the sample data into sample num, tissue, and genotype
	 * @param A single line of sample data
	 * @return a SampleInfo object with num, tissue and geno initalized
	 */
	//Break down the line into sample number, genotype, tissuetype
	public static SampleInfo parseSampleLine(String sampleLine){
		SampleInfo temp = new SampleInfo();
		StringTokenizer tokenizeSamples = new StringTokenizer(sampleLine, ", ");
		double sampleNum = Double.parseDouble(tokenizeSamples.nextToken());
		temp.setSampleNum(sampleNum);
		temp.setGeno(tokenizeSamples.nextToken());
		temp.setTissue(tokenizeSamples.nextToken());
		return temp;
	}
}
